from hpprime import eval
from cursors_and_mark import Curs
from micropython import const
from wr_graphics import wr_mouse, wr_rect_p, wr_txtout_p, wr_invert_p, wr_getpix_p, wr_dimgrob_p, wr_triangle_p
from math import cos, sin, radians
from constantes import GR_PAL
from input import Input


class CenterColor():
    WHITE = const(0)
    BLACK = const(1)

    @staticmethod
    def rvb_center(color):
        if color == CenterColor.WHITE:
            return 255, 255, 255
        if color == CenterColor.BLACK:
            return 0, 0, 0

    @staticmethod
    def swap(color):
        if color == CenterColor.WHITE:
            return CenterColor.BLACK
        if color == CenterColor.BLACK:
            return CenterColor.WHITE


class Palette():
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(
                Palette, cls).__new__(cls)
            # Initialisation des attributs
            cls._instance.initialized = False

        return cls._instance

    def __init__(self):
        if self.initialized:
            return
        self.ra = 110
        self.X = 175
        self.Y = 112
        self.nbt = 10
        self._show = False
        self.edgecolor = 0x0
        self.old_edge_color = self.edgecolor
        self.bgcolor = 0xffffff
        self.old_bg_color = self.bgcolor
        self.action = None
        self.curs = Curs(-1, -1)
        # evite le depassement memoire sur la calculatrice
        self.mire_created = False
        self.center_color = CenterColor.WHITE
        self.userMoved = False
        self.initialized = True

    @property
    def show(self):
        return self._show

    @show.setter
    def show(self, new_show):
        self._show = new_show
        if new_show:
            self.old_edge_color = self.edgecolor
            self.old_bg_color = self.bgcolor
            self.userMoved = False

    def display(self, gr):
        if not self.mire_created:
            self.mire_color_c(gr)
            self.mire_created = True

        return gr

    def update(self):
        x1, y1 = self.curs.get_xy()
        kb = Input().kb
        f1, f2 = wr_mouse()
        self.action(GR_PAL, self.curs)

        wr_rect_p(GR_PAL, 5, 17, 5+52,
                  17+52, 0x0, 0, self.edgecolor, 0)
        wr_rect_p(GR_PAL, 5, 5, 60,
                  13, 0x808080, 0, 0x808080, 0)
        wr_txtout_p("# "+str(hex(self.edgecolor)).upper()
                    [2:8], GR_PAL, 5, 4, 2, 0x0, 120, 0x808080)

        wr_rect_p(GR_PAL, 5, 84, 5+52,
                  84+52, 0x0, 0, self.bgcolor, 0)
        wr_rect_p(GR_PAL, 5, 72, 60,
                  80, 0x808080, 0, 0x808080, 0)
        wr_txtout_p("# "+str(hex(self.bgcolor)).upper()
                    [2:8], GR_PAL, 5, 72, 2, 0x0, 120, 0x808080)

        if len(f1) > 0 and len(f2) == 0:
            self.curs.set_xy(f1[0], f1[1])
            self.userMoved = True

        elif len(f1) > 0 and len(f2) > 0:
            self.curs.set_xy(f1[0], f1[1])
            self.curs.set_xy(f2[0], f2[1])
            self.userMoved = True

        if (kb == 2**30):
            # pick_edge ou pick_bgcolor
            self.action(GR_PAL, self.curs)
            if self.center_color == CenterColor.BLACK:
                wr_invert_p(
                    GR_PAL, 0, 0, 320, 240)
            self.curs = Curs(-1, -1)
            self.show = False
        elif kb == 2**1 or kb == 2**6:
            self.curs = Curs(-1, -1)
            self.edgecolor = self.old_edge_color
            self.bgcolor = self.old_bg_color
        elif kb == 2**23:
            self.center_color = CenterColor.swap(
                self.center_color)
            wr_invert_p(
                GR_PAL, 0, 0, 320, 240)
        elif kb == 2**39 and x1 < 319:
            self.curs.set_xy(x1+1, y1)

        elif kb == 2**37 and x1 > 0:
            self.curs.set_xy(x1-1, y1)

        elif kb == 2**43 and y1 < 239:
            self.curs.set_xy(x1, y1+1)

        elif kb == 2**33 and y1 > 0:
            self.curs.set_xy(x1, y1-1)

        elif kb == 2**32 and x1 > 0 and y1 > 0:
            self.curs.set_xy(x1-1, y1-1)

        elif kb == 2**42 and x1 > 0 and y1 < 239:
            self.curs.set_xy(x1-1, y1+1)

        elif kb == 2**44 and x1 < 319 and y1 < 239:
            self.curs.set_xy(x1+1, y1+1)

        elif kb == 2**34 and x1 < 319 and y1 > 0:
            self.curs.set_xy(x1+1, y1-1)

    def pick_edgecolor(self, gr: int, cur: Curs):
        if self.userMoved:
            self.edgecolor = wr_getpix_p(
                gr, cur.get_x(), cur.get_y())

    def pick_bgcolor(self, gr: int, cur: Curs):
        if self.userMoved:
            self.bgcolor = wr_getpix_p(
                gr, cur.get_x(), cur.get_y())

    def mire_color_c(self, gr):
        start, end, pas = 0, 360, self.nbt
        r = int(self.ra)
        x, y = int(self.X), int(self.Y)
        r1, v1, b1 = CenterColor.rvb_center(
            self.center_color)
        c1 = str(
            int((r1*(2**16)+(v1*(2**8))+b1)))
        lr = []
        lv = []
        lb = []
        c2 = 0
        c3 = 0
        xc1, xc2, xc3 = 0, 0, 0
        yc1, yc2, yc3 = 0, 0, 0
        dim_rect_color = 24

        wr_dimgrob_p(
            gr, 320, 240, 0x808080)

        for k in range(0, 256, 1):
            lr.append(0)
            lv.append(255)
            lb.append(k)
        for h in range(255, -1, -1):
            lr.append(0)
            lv.append(h)
            lb.append(255)
        for m in range(0, 256, 1):
            lr.append(m)
            lv.append(0)
            lb.append(255)
        for n in range(255, -1, -1):
            lr.append(255)
            lv.append(0)
            lb.append(n)
        for i in range(0, 256, 1):
            lr.append(255)
            lv.append(i)
            lb.append(0)
        for j in range(255, -1, -1):
            lr.append(j)
            lv.append(255)
            lb.append(0)

        for i in range(start, end, pas):
            xc1 = str(x)
            yc1 = str(y)
            xc2 = str(x+r*cos(radians(i)))
            yc2 = str(y+r*sin(radians(i)))
            xc3 = str(
                x+r*cos(radians(i)+radians(pas)))
            yc3 = str(
                y+r*sin(radians(i)+radians(pas)))
            c2 = str(int((lr[round(i*(64/15)-1)]*2**16) +
                         (lv[round(i*(64/15)-1)]*2**8)+lb[round(i*(64/15)-1)]))
            c3 = str(int((lr[round(((i+pas)*(64/15)-1))]*2**16) +
                         (lv[round(((i+pas)*(64/15)-1))]*2**8)+lb[round(((i+pas)*(64/15)-1))]))
            eval("TRIANGLE_P(G"+str(gr)+",{"+xc1+","+yc1+","+c1 +
                 "},{"+xc2+","+yc2+","+c2+"},{"+xc3+","+yc3+","+c3+",0})")

        wr_rect_p(GR_PAL, 292, 3, 292+dim_rect_color, 3 +
                  dim_rect_color, 0xff0000, 0, 0xff0000, 0)
        wr_rect_p(GR_PAL, 292, 3+dim_rect_color, 292+dim_rect_color,
                  3+2*dim_rect_color, 0xff8000, 0, 0xff8000, 0)
        wr_rect_p(GR_PAL, 292, 3+2*dim_rect_color, 292+dim_rect_color,
                  3+3*dim_rect_color, 0xffff00, 0, 0xffff00, 0)
        wr_rect_p(GR_PAL, 292, 3+3*dim_rect_color, 292+dim_rect_color,
                  3+4*dim_rect_color, 0x00ff00, 0, 0x00ff00, 0)
        wr_rect_p(GR_PAL, 292, 3+4*dim_rect_color, 292+dim_rect_color,
                  3+5*dim_rect_color, 0x00ffff, 0, 0x00ffff, 0)
        wr_rect_p(GR_PAL, 292, 3+5*dim_rect_color, 292 +
                  dim_rect_color, 3+6*dim_rect_color, 0xff, 0, 0xff, 0)
        wr_rect_p(GR_PAL, 292, 3+6*dim_rect_color, 292+dim_rect_color,
                  3+7*dim_rect_color, 0xff00ff, 0, 0xff00ff, 0)
        wr_rect_p(GR_PAL, 292, 3+7*dim_rect_color, 292 +
                  dim_rect_color, 3+8*dim_rect_color, 0x0, 0, 0x0, 0)
        wr_rect_p(GR_PAL, 292, 3+8*dim_rect_color, 292+dim_rect_color,
                  3+9*dim_rect_color, 0xffffff, 0, 0xffffff, 0)
        wr_triangle_p(GR_PAL, 32, 224, 0x0, 32, 240,
                      0x0, 32+255, 224, 0xffffff, 0)
        wr_triangle_p(GR_PAL, 32, 240, 0x0, 32+255, 224,
                      0xffffff, 32+256, 240, 0xffffff, 0)
        lr = 0
        lv = 0
        lb = 0
