from graphics_shape import Shape
from wr_graphics import wr_pixon_p, wr_rect_p

class Mark:
    def __init__(self, x: int, y: int, show: bool):
        self.x = x
        self.y = y
        self.__show = show

    @property
    def show(self):
        return self.__show

    @show.setter
    def show(self, new_show):
        if not new_show:
            Selection().validated = False
            Selection().show = False
        self.__show = new_show

    def display(self, gr: int) -> int:
        self.mark(gr)
        return gr

    def mark(self, gr):
        if self.show:
            wr_pixon_p(
                gr, self.x-1, self.y-1, 0x0, 0)
            wr_pixon_p(
                gr, self.x-2, self.y-2, 0xffffff, 0)
            wr_pixon_p(
                gr, self.x-3, self.y-3, 0x0, 0)
            wr_pixon_p(
                gr, self.x+1, self.y+1, 0x0, 0)
            wr_pixon_p(
                gr, self.x+2, self.y+2, 0xffffff, 0)
            wr_pixon_p(
                gr, self.x+3, self.y+3, 0x0, 0)
            Shape.invert_pixel(
                gr, self.x, self.y)
            wr_pixon_p(
                gr, self.x+1, self.y-1, 0x0, 0)
            wr_pixon_p(
                gr, self.x+2, self.y-2, 0xffffff, 0)
            wr_pixon_p(
                gr, self.x+3, self.y-3, 0x0, 0)
            wr_pixon_p(
                gr, self.x-1, self.y+1, 0x0, 0)
            wr_pixon_p(
                gr, self.x-2, self.y+2, 0xffffff, 0)
            wr_pixon_p(
                gr, self.x-3, self.y+3, 0x0, 0)


class Curs:
    def __init__(self, x: int, y: int):
        self.__x = x
        self.__y = y
        self.__xprev = x
        self.__yprev = y

    @property
    def x(self):
        return self.__x

    @x.setter
    def x(self, new_x):
        self.__xprev = self.__x
        self.__x = new_x

    @property
    def y(self):
        return self.__y

    @y.setter
    def y(self, new_y):
        self.__yprev = self.__y
        self.__y = new_y

    def set_xy(self, x, y):
        self.x = x
        self.y = y

    def get_xprev(self):
        return self.__xprev

    def get_yprev(self):
        return self.__yprev

    def get_x(self):
        return self.__x

    def get_y(self):
        return self.__y

    def get_xy(self):
        return self.__x, self.__y

    def get_xyprev(self):
        return self.__xprev, self.__yprev

    def display(self, gr: int) -> int:
        self.curs(gr)
        return gr

    def curs(self, gr):
        wr_pixon_p(
            gr, self.__x-1, self.__y, 0x0, 0)
        wr_pixon_p(
            gr, self.__x-2, self.__y, 0xffffff, 0)
        wr_pixon_p(
            gr, self.__x-3, self.__y, 0x0, 0)
        wr_pixon_p(
            gr, self.__x+1, self.__y, 0x0, 0)
        wr_pixon_p(
            gr, self.__x+2, self.__y, 0xffffff, 0)
        wr_pixon_p(
            gr, self.__x+3, self.__y, 0x0, 0)
        Shape.invert_pixel(
            gr, self.__x, self.__y)
        wr_pixon_p(
            gr, self.__x, self.__y-1, 0x0, 0)
        wr_pixon_p(
            gr, self.__x, self.__y-2, 0xffffff, 0)
        wr_pixon_p(
            gr, self.__x, self.__y-3, 0x0, 0)
        wr_pixon_p(
            gr, self.__x, self.__y+1, 0x0, 0)
        wr_pixon_p(
            gr, self.__x, self.__y+2, 0xffffff, 0)
        wr_pixon_p(
            gr, self.__x, self.__y+3, 0x0, 0)


class Selection:
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(
                Selection, cls
            ).__new__(cls)
            # Initialisation des attributs
            cls._instance.initialized = (
                False
            )
        return cls._instance

    def __init__(self, mk: Mark = None, curs: Curs = None):
        if self.initialized:
            return
        self.mark = Mark(mk.x, mk.y, mk.show)
        self.curs = curs
        self.mark2 = Mark(mk.x, mk.y, False)
        self.show = False
        self.validated = False
        self.initialized = True

    def validate(self):
        self.mark2.x = self.curs.x
        self.mark2.y = self.curs.y
        self.validated = True

    def display(self, gr: int) -> int:
        if self.show:
            if self.validated:
                Shape.invert_box_sel(gr,
                    self.mark.x,
                    self.mark.y,
                    self.mark2.x,
                    self.mark2.y)
                """wr_rect_p(
                    gr,
                    self.mark.x,
                    self.mark.y,
                    self.mark2.x,
                    self.mark2.y,
                    0x0,
                    128,
                )"""
            else:
                Shape.invert_box_sel(gr,
                    self.mark.x,
                    self.mark.y,
                    self.curs.x,
                    self.curs.y)   
                """wr_rect_p(
                    gr,
                    self.mark.x,
                    self.mark.y,
                    self.curs.x,
                    self.curs.y,
                    0x0,
                    128,
                )"""
        return gr
