import gc
import math
from hpprime import eval, strblit2

from wr_graphics import (
    wr_fill_zone,
    wr_getpix_p,
    wr_invert_p,
    wr_subgrob_p,
    wr_triangle_p,
    wr_line_p,
    wr_rect_p,
    wr_pixon_p,
    wr_arc_p,
    wr_txtout_nbg_p,
    wr_txtout_p,
    wr_graphic_to_display,
)
from math import sqrt, cos, sin, radians
from constantes import GR_AFF, GR_DRAW, GR_ICO, GR_LAYERS, GR_TMP, GR_VARIOUS


class Shape:
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(
                Shape, cls
            ).__new__(cls)
            # Initialisation des attributs
            cls._instance.initialized = (
                False
            )

        return cls._instance

    def __init__(self):
        if self.initialized:
            return
        self.gr = GR_TMP
        self.edgecolor = 0x0
        self.bgcolor = 0xFFFFFF
        self.ca_edgecolor = 0
        self.ca_bgcolor = 0
        self.tool = self.nop
        self.polyline_prev = 0
        self.initialized = True
        self.txt = ""
        self.police = 1
        self.cache_gr_layers = False

    def polyline(self, mark, curs):
        if mark.show:
            mark.show = False
        if self.polyline_prev != 0:
            wr_line_p(
                GR_DRAW,
                curs.get_xprev(),
                curs.get_yprev(),
                curs.get_x(),
                curs.get_y(),
                self.edgecolor,
                self.ca_edgecolor,
            )
        else:
            wr_pixon_p(
                GR_DRAW,
                curs.get_x(),
                curs.get_y(),
                self.edgecolor,
                self.ca_edgecolor,
            )
        self.polyline_prev = 1

    def point(self, mark, curs):
        if mark.show:
            mark.show = False
        wr_pixon_p(
            GR_DRAW,
            curs.get_x(),
            curs.get_y(),
            self.edgecolor,
            self.ca_edgecolor,
        )

    def line(self, mark, curs):
        wr_line_p(
            self.gr,
            mark.x,
            mark.y,
            curs.get_x(),
            curs.get_y(),
            self.edgecolor,
            self.ca_edgecolor,
        )

    def rect(self, mark, curs):
        wr_rect_p(
            self.gr,
            mark.x,
            mark.y,
            curs.get_x(),
            curs.get_y(),
            self.edgecolor,
            self.ca_edgecolor,
        )

    def shade_bg_h(self, mark, curs):
        Shape.shade_h_rect_p(
            self.gr,
            mark.x,
            mark.y,
            curs.get_x(),
            curs.get_y(),
            self.edgecolor,
            self.bgcolor,
            self.ca_bgcolor
        )

    def shade_bg_v(self, mark, curs):
        Shape.shade_v_rect_p(
            self.gr,
            mark.x,
            mark.y,
            curs.get_x(),
            curs.get_y(),
            self.edgecolor,
            self.bgcolor,
            self.ca_bgcolor
        )

    def shade_rect_p(self, mark, curs):
        Shape.shade_rect_rect_p(
            self.gr,
            mark.x,
            mark.y,
            curs.get_x(),
            curs.get_y(),
            self.edgecolor,
            self.bgcolor,
            self.ca_bgcolor
        )

    def fill_rect(self, mark, curs):
        wr_rect_p(
            self.gr,
            mark.x,
            mark.y,
            curs.get_x(),
            curs.get_y(),
            self.edgecolor,
            self.ca_edgecolor,
            self.bgcolor,
            self.ca_bgcolor,
        )

    def circle(self, mark, curs):
        r1 = r2 = int(
            sqrt(
                (curs.get_x() - mark.x)
                ** 2
                + (
                    curs.get_y()
                    - mark.y
                )
                ** 2
            )
        )
        wr_arc_p(
            self.gr,
            mark.x,
            mark.y,
            r1,
            r2,
            0,
            360,
            self.edgecolor,
            self.ca_edgecolor,
        )

    def fill_circle(self, mark, curs):
        r1 = r2 = int(
            sqrt(
                (curs.get_x() - mark.x)
                ** 2
                + (
                    curs.get_y()
                    - mark.y
                )
                ** 2
            )
        )
        wr_arc_p(
            self.gr,
            mark.x,
            mark.y,
            r1,
            r2,
            0,
            360,
            self.edgecolor,
            self.ca_edgecolor,
            self.bgcolor,
            self.ca_bgcolor,
        )

    def shade_circle(self, mark, curs):
        Shape.shade_circle_p(
            self.gr,
            mark.x,
            mark.y,
            curs.get_x(),
            curs.get_y(),
            self.edgecolor,
            self.bgcolor,
            self.ca_bgcolor
        )
        gc.collect()

    def text(self, mark, curs):
        wr_txtout_p(
            self.txt,
            self.gr,
            curs.get_x(),
            curs.get_y(),
            self.police,
            self.edgecolor,
            511,
            self.bgcolor,
        )

    def text_nbg(self, mark, curs):
        wr_txtout_nbg_p(
            self.txt,
            self.gr,
            curs.get_x(),
            curs.get_y(),
            self.police,
            self.edgecolor,
            511
        )

    def fill_zone(self, mark, curs):
        Shape.wait_timer()
        wr_fill_zone(
            GR_DRAW, curs.x, curs.y, self.edgecolor)
        self.tool = self.nop

    def stamp(self, mark, curs):
        from cursors_and_mark import Selection
        sel = Selection()
        sx = sel.mark.x if sel.mark.x < sel.mark2.x else sel.mark2.x
        sy = sel.mark.y if sel.mark.y < sel.mark2.y else sel.mark2.y
        width = abs(sel.mark2.x - sel.mark.x) + 1
        height = abs(sel.mark2.y - sel.mark.y) + 1
        strblit2(GR_TMP, curs.x, curs.y, width,height, GR_DRAW, sx, sy, width, height)
    
    def mirror_hrt(self, mark, curs):
        from cursors_and_mark import Selection
        sel = Selection()
        sx = sel.mark.x if sel.mark.x < sel.mark2.x else sel.mark2.x
        sy = sel.mark.y if sel.mark.y < sel.mark2.y else sel.mark2.y
        width = abs(sel.mark2.x - sel.mark.x) + 1
        height = abs(sel.mark2.y - sel.mark.y) + 1
        if not self.cache_gr_layers:
            Shape.wait_timer()
            Shape.mirror_h(GR_LAYERS, sx, sy, sx + width - 1, sy + height - 1)
            self.cache_gr_layers = True
        strblit2(GR_TMP, curs.x, curs.y, width,
                 height, GR_LAYERS, sx, sy, width, height)

    def mirror_vrt(self, mark, curs):
        from cursors_and_mark import Selection
        sel = Selection()
        sx = sel.mark.x if sel.mark.x < sel.mark2.x else sel.mark2.x
        sy = sel.mark.y if sel.mark.y < sel.mark2.y else sel.mark2.y
        width = abs(sel.mark2.x - sel.mark.x) + 1
        height = abs(sel.mark2.y - sel.mark.y) + 1
        if not self.cache_gr_layers:
            strblit2(GR_LAYERS, sx, sy, width,
                    height, GR_TMP, sx, sy, width, height)
            Shape.wait_timer()
            Shape.mirror_v(GR_LAYERS, sx, sy, sx + width - 1, sy + height - 1)
            self.cache_gr_layers = True
        strblit2(GR_TMP, curs.x, curs.y, width,
                 height, GR_LAYERS, sx, sy, width, height)
        


    @staticmethod
    def wait_timer():
        strblit2(GR_AFF, 141, 101, 37, 37, GR_ICO, 74, 185, 37, 37)

    def nop(self, mark, curs):
        pass

    def validate(self, gr_dest):
        self.tool = self.nop
        wr_rect_p(GR_VARIOUS, 0, 0, 319, 239, 0xFFFFFF, 255) #ToDo
        self.cache_gr_layers = False
        wr_graphic_to_display(
            gr_dest, 0, 0, self.gr
        )

    @staticmethod
    def shade_v_rect_p(
            gr, x1, y1, x2, y2, c1, c2, ca_global):
        wr_triangle_p(
            gr,
            x1,
            y1,
            c1,
            x2,
            y1,
            c2,
            x2,
            y2,
            c2,
            ca_global
        )
        wr_triangle_p(
            gr,
            x1,
            y1,
            c1,
            x1,
            y2,
            c1,
            x2,
            y2,
            c2,
            ca_global
        )
        return

    @staticmethod
    def shade_h_rect_p(
        gr, x1, y1, x2, y2, c1, c2, ca_global
    ):
        wr_triangle_p(
            gr,
            x1,
            y1,
            c1,
            x2,
            y1,
            c1,
            x2,
            y2,
            c2,
            ca_global
        )
        wr_triangle_p(
            gr,
            x1,
            y1,
            c1,
            x1,
            y2,
            c2,
            x2,
            y2,
            c2,
            ca_global
        )
        return

    @staticmethod
    def shade_rect_rect_p(
        gr, x1, y1, x2, y2, c1, c2, ca_global
    ):
        wr_triangle_p(
            gr,
            x1,
            y1,
            c1,
            x1 + (x2 - x1) / 2,
            y2 - (y2 - y1) / 2,
            c2,
            x1,
            y2,
            c1,
            ca_global
        )
        wr_triangle_p(
            gr,
            x1,
            y1,
            c1,
            x1 + (x2 - x1) / 2,
            y2 - (y2 - y1) / 2,
            c2,
            x2,
            y1,
            c1,
            ca_global
        )
        wr_triangle_p(
            gr,
            x2,
            y1,
            c1,
            x1 + (x2 - x1) / 2,
            y2 - (y2 - y1) / 2,
            c2,
            x2,
            y2,
            c1,
            ca_global
        )
        wr_triangle_p(
            gr,
            x1,
            y2,
            c1,
            x1 + (x2 - x1) / 2,
            y2 - (y2 - y1) / 2,
            c2,
            x2,
            y2,
            c1,
            ca_global
        )
        return

    @staticmethod
    def shade_circle_p(
        gr, x1, y1, x2, y2, c1, c2, ca_global
    ):
        for i in range(0, 360, 8):
            eval(
                "TRIANGLE_P(G"
                + str(gr)
                + ","
                + str(x1)
                + ","
                + str(y1)
                + ","
                + str(
                    x1
                    + int(
                        sqrt(
                            (x2 - x1) ** 2
                            + (y2 - y1) ** 2
                        )
                    )
                    * cos(radians(i))
                )
                + ","
                + str(
                    y1
                    + int(
                        sqrt(
                            (x2 - x1) ** 2
                            + (y2 - y1) ** 2
                        )
                    )
                    * sin(radians(i))
                )
                + ","
                + str(
                    x1
                    + int(
                        sqrt(
                            (x2 - x1) ** 2
                            + (y2 - y1) ** 2
                        )
                    )
                    * cos(
                        radians(i)
                        + radians(8)
                    )
                )
                + ","
                + str(
                    y1
                    + int(
                        sqrt(
                            (x2 - x1) ** 2
                            + (y2 - y1) ** 2
                        )
                    )
                    * sin(
                        radians(i)
                        + radians(8)
                    )
                )
                + ","
                + str(c2)
                + ","
                + str(c1)
                + ","
                + str(c1)
                + ","
                + str(ca_global) + ")"
            )

    @staticmethod
    def invert_line_p(
        gr,
        x1: int,
        y1: int,
        x2: int,
        y2: int,
    ):
        dx = x2 - x1
        dy = y2 - y1
        if abs(dx) > abs(dy):
            step = abs(dx)
        else:
            step = abs(dy)
        xi = dx / step
        yi = dy / step
        x, y = x1, y1
        for _ in range(step):
            Shape.invert_pixel(
                gr, round(x), round(y)
            )
            x += xi
            y += yi
        return

    @staticmethod
    def invert_pixel(gr, x1: int, y1: int):
        wr_invert_p(gr, x1, y1, x1, y1)
        return

    @staticmethod
    def invert_filled_circle(
        gr, x: int, y: int, r: int
    ):
        for dy in range(-r, r + 1):
            for dx in range(-r, r + 1):
                if (
                    dx**2 + dy**2
                    <= r**2
                ):
                    Shape.invert_pixel(
                        gr, x + dx, y + dy
                    )
        return

    @staticmethod
    def mirror_h(gr, x1: int, y1: int, x2: int, y2: int):
        width = abs(x2-x1) + 1
        height= abs(y2-y1) + 1
        dimx = math.ceil(width/2)
        for i in range(0, width):
            strblit2(GR_LAYERS, x2 - i, y1, 1, height, 
                     GR_TMP, x1 + i , y1, 1, height)
        return gr

    
    @staticmethod
    def mirror_v(gr, x1: int, y1: int, x2: int, y2: int):
        width = abs(x2-x1) + 1
        height= abs(y2-y1) + 1
        dimy = math.ceil(height/2)
        for i in range(0, height):
            strblit2(GR_LAYERS, x1, y2 - i, width, 1, 
                     GR_TMP, x1 , y1 + i, width, 1)
        return gr

    
    @staticmethod
    def invert_box_sel(gr, x1: int, y1: int, x2: int, y2: int):
        rx1, ry1, rx2, ry2 = Shape.order_rect(x1, y1, x2, y2)
        wr_invert_p(gr, rx1, ry1, rx2, ry1)
        wr_invert_p(gr, rx2, ry1, rx2, ry2)
        wr_invert_p(gr, rx1, ry2, rx2, ry2)
        wr_invert_p(gr, rx1, ry1, rx1, ry2)

    @staticmethod
    def order_rect(x1: int, y1: int, x2: int, y2: int):
        rx1 = x1 if x1 < x2 else x2
        ry1 = y1 if y1 < y2 else y2
        rx2 = x2 if x1 < x2 else x1
        ry2 = y2 if y1 < y2 else y1

        return rx1, ry1, rx2, ry2