from about_help import About_Help
from wr_os import (
    open_file,
    save_file,
)
from gui_canvas import Canvas
from micropython import const
from color_palette import Palette
from input import Input
from constantes import GR_GUI, GR_TMP, GR_AFF, GR_ICO, ICO_WIDTH, GR_CURS, GR_DRAW, GR_PAL
from wr_graphics import (
    wr_dimgrob_p,
    wr_graphic_to_display,
    wr_mouse,
    wr_subgrob_p,
    wr_rect_p,
    wr_txtout_p
)
from cursors_and_mark import Curs

class IcoPreriquisite:
    NOTHING = const(1)
    MARK = const(2)
    SELECTION = const(3)

class GuiComponent:
    def __init__(self):
        # Reference container component
        self.master = None
        # relative to master container component
        self.x = 0
        # relative to master container component
        self.y = 0
        self.width = 0
        self.height = 0

    def draw(self):
        pass

    def getPos(self):
        if self.master is not None:
            return (
                self.master.x + self.x,
                self.master.y + self.y,
            )
        else:
            return self.x, self.y

    def setPos(self, x, y):
        self.x = x
        self.y = y

    def getSize(self):
        return self.width, self.height

    def setSize(self, w, h):
        self.width = w
        self.height = h

    def update(self, curs):
        pass


class GuiContainer(GuiComponent):
    def __init__(self):
        super().__init__()
        self.components = []

    def add(self, component):
        component.master = self
        self.components.append(
            component
        )

    def draw(self):
        wr_rect_p(
            GR_GUI,
            self.x,
            self.y,
            self.x + self.width,
            self.y + self.height,
            0x808080,
            0,
            0x808080,
            0,
        )


class GuiButton(GuiComponent):
    def __init__(self):
        super().__init__()
        # function to call when clicked
        self.action = None
        self.text = ""
        self.label = ""


class GuiIcon(GuiButton):
    def __init__(self):
        super().__init__()
        self.picture = ()  # x, y, w, h
        self.need_mark = False
        self.need_selection = False
        self.icon_active = True

    def grey_icon(self):
        wr_rect_p(
            GR_TMP,
            0,
            0,
            ICO_WIDTH,
            ICO_WIDTH,
            0x0,
            128,
            0x0,
            128,
        )

    def draw(self):
        wr_subgrob_p(
            GR_ICO,
            self.picture[0],
            self.picture[1],
            self.picture[0]
            + self.picture[2],
            self.picture[1]
            + self.picture[3],
            GR_TMP,
        )
        if not self.icon_active:
            self.grey_icon()

        x, y = self.getPos()
        wr_graphic_to_display(
            GR_GUI, x, y, GR_TMP
        )
        wr_dimgrob_p(
            GR_TMP, 320, 240, 0xFFFFFF
        )
        # ToDo dessin de l'icône

    def update(self, curs):
        x, y = self.getPos()
        w, h = self.getSize()
        if (
            x
            <= int(curs.get_x())
            <= x + w
            and y
            <= int(curs.get_y())
            <= y + h
        ):
            exec(self.action)


class GuiGrid(
    GuiContainer
):  # heritage de Guiconter
    def __init__(self):
        super().__init__()
        self.page = 0
        self.right_page_icon = None
        self.left_page_icon = None
        self.__grid_size = 0

    def draw(self):
        """
        Draw grid of icons
        """
        super().draw()  # demande à la classe mere de se dessiner (zone grise)
        #self.right_page_icon.draw()
        #self.left_page_icon.draw()
        # a chaque icon, il demande de de se dessiner
        for i in range(
            0, len(self.components)
        ):
            if i == self.__grid_size:
                break

            if (
                self.components[
                    i
                ].need_mark):
                self.components[
                    i
                ].icon_active = Canvas().mk.show
            elif (
                self.components[
                    i
                ].need_selection):
                self.components[
                    i
                ].icon_active = Canvas().select.validated
            else:
                self.components[
                    i
                ].icon_active = True
            self.components[i].draw()

    @staticmethod
    def getPictureSize(picture):
        return picture[2], picture[3]

    def init(self, section_icons):
        grid_org_x = ICO_WIDTH + 2
        grid_org_y = 0
        grid_width = self.width - 2 * (
            ICO_WIDTH + 2
        )
        grid_height = self.height
        self.right_page_icon = (
            GuiMenu.init_icon(
                self.width - 37,
                5,
                ICO_WIDTH,
                ICO_WIDTH,
                (37, 37, 37, 185),
                self,
                "",
            )
        )
        self.left_page_icon = GuiMenu.init_icon(
            1,
            5,
            ICO_WIDTH,
            ICO_WIDTH,
            (0, 37, 37, 185),
            self,
            "",
        )
        grid_line_capacity = int(
            grid_width / ICO_WIDTH
        )
        self.__grid_size = (
            grid_line_capacity
            * int(
                grid_height
                / (ICO_WIDTH)
            )
        )
        for i in range(
            0, len(section_icons)
        ):
            ico = GuiIcon()
            ico.x = grid_org_x + int(
                i % grid_line_capacity
            ) * (ICO_WIDTH + 3)
            ico.y = grid_org_y + int(
                i
                % self.__grid_size
                / grid_line_capacity
            ) * (ICO_WIDTH + 3)
            ico.label = section_icons[
                i
            ][0]
            ico.picture = section_icons[
                i
            ][1]
            (
                ico.width,
                ico.height,
            ) = GuiGrid.getPictureSize(
                ico.picture
            )
            action_method = (
                section_icons[i][2]
            )
            ico.action = action_method
            ico.need_mark = (
                section_icons[i][3] == IcoPreriquisite.MARK
            )
            ico.need_selection = (
                section_icons[i][3] == IcoPreriquisite.SELECTION
            )
            self.add(ico)

    def update(self, curs):
        for icon in self.components:
            if (
                Canvas().mk.show
                or not (icon.need_mark or icon.need_selection)
            ):
                icon.update(curs)


class GuiMenu(GuiContainer):
    def __init__(self):
        super().__init__()
        self.sections = (
            []
        )  # type [] GuiContainer
        self.selected_section = 1
        self.left_icon = None
        self.right_icon = None
        self._curs = Curs(-1, -1)
        self._show = False

    @property
    def show(self):
        return self._show

    @show.setter
    def show(self, new_show):
        if not new_show:
            self.curs.set_xy(-1, -1)
        self._show = new_show

    @property
    def curs(self):
        return self._curs

    @curs.setter
    def curs(self, new_curs):
        self._curs = new_curs

    def draw(self):
        super().draw()
        #self.left_icon.draw()
        #self.right_icon.draw()
        for icon in self.components:
            icon.draw()
        # demande à la section selection de se dessiner
        self.sections[
            self.selected_section
        ].draw()

    def init(self, gui_icons):
        self.left_icon = (
            GuiMenu.init_icon(
                1,
                1,
                ICO_WIDTH,
                ICO_WIDTH,
                (
                    0,
                    0,
                    ICO_WIDTH,
                    ICO_WIDTH,
                ),
                self,
                "",
            )
        )
        self.right_icon = (
            GuiMenu.init_icon(
                self.width - 37,
                1,
                ICO_WIDTH,
                ICO_WIDTH,
                (
                    ICO_WIDTH + 1,
                    0,
                    ICO_WIDTH,
                    ICO_WIDTH,
                ),
                self,
                "",
            )
        )
        for menu_entry in gui_icons:
            section_container = (
                GuiGrid()
            )
            # positionnnement de la zone grise
            section_container.setPos(
                2, 41
            )  # 14, 38)
            section_container.setSize(
                314, 196
            )  # 292, 181)
            # section_container.setSize(218, 181)  # zone grise pour les icones
            section_container.init(
                menu_entry[1]
            )
            self.sections.append(
                section_container
            )

            space = (
                self.width
                - 40
                - 37
                - (50 + 40 + 61 + 37)
            ) / 5
            icon = GuiMenu.init_icon(
                40 + space,
                4,
                50,
                ICO_WIDTH,
                (74, 223, 50, 30),
                self,
                "Gui().selectedSection(0)",
            )
            self.add(icon)
            icon = GuiMenu.init_icon(
                90 + 2 * space,
                4,
                40,
                ICO_WIDTH,
                (125, 223, 40, 30),
                self,
                "Gui().selectedSection(1)",
            )
            self.add(icon)
            icon = GuiMenu.init_icon(
                130 + 3 * space,
                4,
                61,
                ICO_WIDTH,
                (167, 223, 61, 30),
                self,
                "Gui().selectedSection(2)",
            )
            self.add(icon)
            icon = GuiMenu.init_icon(
                191 + 4 * space,
                4,
                ICO_WIDTH + 1,
                ICO_WIDTH,
                (229, 223, 37, 30),
                self,
                "Gui().selectedSection(3)",
            )
            self.add(icon)

    @staticmethod
    def init_icon(
        x,
        y,
        w,
        h,
        picture,
        master,
        action,
    ) -> GuiIcon:
        icon = GuiIcon()
        icon.setPos(x, y)
        icon.setSize(w, h)
        icon.picture = picture
        icon.master = master
        icon.action = action
        return icon

    def update(self):
        f1, f2 = wr_mouse()
        if len(f1) > 0 and len(f2) == 0:
            self.curs.set_xy(
                f1[0], f1[1]
            )

        elif (
            len(f1) > 0 and len(f2) > 0
        ):
            self.curs.set_xy(
                f1[0], f1[1]
            )
            self.curs.set_xy(
                f2[0], f2[1]
            )

        self.left_icon.update(self.curs)
        self.right_icon.update(
            self.curs
        )
        for icon in self.components:
            icon.update(self.curs)
        self.sections[
            self.selected_section
        ].update(self.curs)

        """Transparency

        Returns:
            _type_: _description_
        """


_RESOURCE_NAME = "transp.png"


class GuiColorSlider(GuiComponent):
    def __init__(self, color, canal):
        super().__init__()
        self.color = color
        self.canal = canal

    def update(self, curs):
        x, y = self.getPos()
        w, h = self.getSize()
        if (
            x
            < int(curs.get_x())
            <= x + w
            and y
            <= int(curs.get_y())
            <= y + h
        ):
            self.canal = 255 - (
                curs.get_x() - x - 1
            )

    def draw(self):
        x, y = self.getPos()
        w, h = self.getSize()
        wr_rect_p(
            GR_GUI,
            x,
            y,
            x + w,
            y + h,
            self.color,
            self.canal,
            self.color,
            self.canal,
        )


class ColorTransparency:
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(
                ColorTransparency, cls
            ).__new__(cls)
            # Initialisation des attributs
            cls._instance.initialized = (
                False
            )

        return cls._instance

    def __init__(self):
        if self.initialized:
            return
        self._show = False
        self.edgecolor = 0x0
        self.bgcolor = 0xFFFFFF
        self.ca_edgecolor = 0
        self.ca_bgcolor = 0
        self.edgecolorSlider = (
            GuiColorSlider(
                self.edgecolor,
                self.ca_edgecolor,
            )
        )
        self.edgecolorSlider.setPos(
            31, 61
        )
        self.edgecolorSlider.setSize(
            256, 56
        )
        self.bgcolorSlider = (
            GuiColorSlider(
                self.bgcolor,
                self.ca_bgcolor,
            )
        )
        self.bgcolorSlider.setPos(
            31, 157
        )
        self.bgcolorSlider.setSize(
            256, 56
        )
        self.curs = Curs(160, 120)
        self.initialized = True

    @property
    def show(self):
        return self._show

    @show.setter
    def show(self, new_show):
        self._show = new_show

    def draw(self, gr):
        self.edgecolorSlider.color = (
            self.edgecolor
        )
        self.bgcolorSlider.color = (
            self.bgcolor
        )
        
        
        open_file(
            GR_GUI, _RESOURCE_NAME
        )
        
        wr_txtout_p(str(255-self.edgecolorSlider.canal),GR_GUI,31,45,1,0x0000FF,320,0xFFFFFF)        
        wr_txtout_p(str(255-self.bgcolorSlider.canal),GR_GUI,31,141,1,0x0000FF,320,0xFFFFFF)
        
        self.edgecolorSlider.draw()
        self.bgcolorSlider.draw()
        wr_rect_p(GR_GUI,255-self.edgecolorSlider.canal-2+32,59,255-self.edgecolorSlider.canal+2+32,59+60,0x000000,0,0xFFFFFF,0)
        wr_rect_p(GR_GUI,255-self.bgcolorSlider.canal-2+32,155,255-self.bgcolorSlider.canal+2+32,155+60,0x000000,0,0xFFFFFF,0)
        return gr

    def update(self):
        kb = Input().kb

        f1, f2 = wr_mouse()
        if len(f1) > 0 and len(f2) == 0:
            self.curs.set_xy(
                f1[0], f1[1]
            )

        elif (
            len(f1) > 0 and len(f2) > 0
        ):
            self.curs.set_xy(
                f1[0], f1[1]
            )
            self.curs.set_xy(
                f2[0], f2[1]
            )

        if kb == 2**30:
            self.ca_edgecolor = (
                self.edgecolorSlider.canal
            )
            self.ca_bgcolor = (
                self.bgcolorSlider.canal
            )
            self.curs.set_xy(160, 120)
            self.show = False

        self.edgecolorSlider.update(
            self.curs
        )
        self.bgcolorSlider.update(
            self.curs
        )



class GuiStatus:
    CANVAS = const(1)
    MENU = const(2)
    PALETTE = const(3)
    TRANSPARENCY = const(4)
    ABOUT_HELP = const(5)


class Gui:
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(
                Gui, cls
            ).__new__(cls)
            # Initialisation des attributs
            cls._instance.initialized = (
                False
            )
        return cls._instance

    def __init__(self):
        if self.initialized:
            return
        # self.func # Choose user functionality
        # self.actions #Gui.current actions
        self.canvas = Canvas()
        self.choices = []
        self.menu = None
        self.transparency = (
            ColorTransparency()
        )
        self.pal = Palette()
        self.initialized = True
        self.status = GuiStatus.CANVAS
        self.about_help = About_Help()

    def init(self, gui_icons):
        open_file(GR_ICO, "icons.png")
        self.menu = GuiMenu()
        self.menu.setPos(2, 2)
        self.menu.setSize(314, 235)
        self.menu.init(gui_icons)

    def mPt(self):
        kb = Input().kb
        if kb == 2**13:
            if (
                self.status
                != GuiStatus.MENU
            ):
                self.status = (
                    GuiStatus.MENU
                )
                self.menu.show = True
            else:
                self.menu.curs.set_xy(
                    -1, -1
                )
                self.menu.show = False
            # self.show_menu = not self.show_menu
        elif kb == 2**1:
            if (
                self.status
                != GuiStatus.PALETTE
            ):
                self.status = (
                    GuiStatus.PALETTE
                )
                self.pal.show = True
                self.pal.action = (
                    self.pal.pick_edgecolor
                )
            else:
                self.pal.show = False
        elif kb == 2**6:
            if (
                self.status
                != GuiStatus.PALETTE
            ):
                self.status = (
                    GuiStatus.PALETTE
                )
                self.pal.show = True
                self.pal.action = (
                    self.pal.pick_bgcolor
                )
            else:
                self.pal.show = False
        elif kb == 2**3:
            if (
                self.status
                != GuiStatus.ABOUT_HELP
            ):
                self.launch_about_help()
            else:
                self.about_help.show = False

        elif kb == 2**27:
            save_file(GR_DRAW, "graphic_current_save.png")
            self.save()
        elif kb == 2**41 + 2**27:
            #save_file(GR_DRAW, "graphic_current_save.png")
            save_file(GR_DRAW, self.canvas.fm.current_file_name)
            self.save_as()
    def draw(self):
        self.mPt()

        if (
            self.status
            == GuiStatus.CANVAS
        ):
            self.canvas.draw()
            self.canvas.update()

        # DRAW->TMP  TMP->CURS CURS->AFF

        # copie grtmp dans gr_mark
        if (
            self.status
            == GuiStatus.PALETTE
        ):
            gr = self.pal.display(
                GR_PAL
            )
            self.pal.update()
            if (
                not self.pal.show
            ):  # on est sorti de Palette
                self.canvas.shp.edgecolor = (
                    self.pal.edgecolor
                )
                self.canvas.shp.bgcolor = (
                    self.pal.bgcolor
                )
                self.status = (
                    GuiStatus.CANVAS
                )
            else:
                wr_graphic_to_display(
                    GR_CURS, 0, 0, gr
                )
                if self.pal.userMoved:
                    self.pal.curs.display(
                        GR_CURS
                    )
                wr_graphic_to_display(
                    GR_AFF,
                    0,
                    0,
                    GR_CURS,
                )

        # The `wr_graphic_to_display(GR_AFF,0,0,GR_Gui.curS)` function is copying the graphic from the `GR_Gui.curS`
        # buffer to the `GR_AFF` buffer. This allows the graphic to be displayed on the screen.
        # wr_graphic_to_display(GR_AFF,0,0,GR_Gui.curS)
        if (
            self.status
            == GuiStatus.MENU
        ):
            # on récupère show AVANT les modifications de menu.draw()
            show = self.menu.show
            # if self.show_menu:
            # demande à sections de se dessiner
            self.menu.draw()
            wr_graphic_to_display(
                GR_CURS, 0, 0, GR_GUI
            )
            self.menu.update()
            wr_graphic_to_display(
                GR_AFF, 0, 0, GR_CURS
            )
            if (
                not show
            ):  # on est sorti de MENU
                wr_graphic_to_display(
                    GR_TMP,
                    0,
                    0,
                    GR_DRAW,
                )
                Input.clear_mouse()
                self.status = (
                    GuiStatus.CANVAS
                )
        # .gui.draw()

        if (
            self.status
            == GuiStatus.TRANSPARENCY
        ):
            if (
                not self.transparency.show
            ):  # on est sorti de Transparency
                self.canvas.shp.ca_edgecolor = (
                    self.transparency.ca_edgecolor
                )
                self.canvas.shp.ca_bgcolor = (
                    self.transparency.ca_bgcolor
                )
                self.status = (
                    GuiStatus.CANVAS
                )
            else:
                self.transparency.draw(
                    GR_GUI
                )
                self.transparency.update()
                wr_graphic_to_display(
                    GR_AFF, 0, 0, GR_GUI
                )
                

        if (
            self.status
            == GuiStatus.ABOUT_HELP
        ):
            if (
                not self.about_help.show
            ):
                self.status = GuiStatus.CANVAS
            else :
                gr = self.about_help.display(GR_TMP)
                self.about_help.update()
                wr_graphic_to_display(
                        GR_AFF, 0, 0, gr
                )


    def selectedSection(self, num):
        self.menu.selected_section = num

    def select_tool(self, tool):
        canvas = Canvas()
        canvas.shp.tool = getattr(
            canvas.shp, tool
        )
        self.menu.show = False

    def select_input_tool(self, tool):
        self.select_tool(tool)
        Canvas().shp.txt = input(
            "\nVotre texte : ")
        Canvas().shp.police = input("\nTaille Police (0-7) : ")
        Input.clear_kb()

    def launch_palette(self, action):
        self.status = GuiStatus.PALETTE
        self.pal.show = True
        self.pal.action = action
        self.menu.show = False
        Input.clear_mouse()
        Input.clear_kb()
        
    def swap_colors(self):
        old_value = self.canvas.shp.edgecolor
        self.canvas.shp.edgecolor = self.canvas.shp.bgcolor
        self.canvas.shp.bgcolor = old_value
        self.pal.edgecolor = self.canvas.shp.edgecolor
        self.pal.bgcolor = self.canvas.shp.bgcolor
        
    def pick_color(self):
        self.canvas.pick_color()
        self.pal.bgcolor = self.canvas.shp.bgcolor
        self.menu.show = False
        
    def launch_transparency(self):
        self.status = (
            GuiStatus.TRANSPARENCY
        )
        self.menu.show = False
        self.transparency.edgecolor = (
            self.canvas.shp.edgecolor
        )
        self.transparency.bgcolor = (
            self.canvas.shp.bgcolor
        )
        self.transparency.ca_edgecolor = (
            self.canvas.shp.ca_edgecolor
        )
        self.transparency.ca_bgcolor = (
            self.canvas.shp.ca_bgcolor
        )
        self.transparency.show = True
        
    
    def launch_zoom(self):
        print("launch zoom")
        if self.menu.show:
            self.canvas.zoom.show = True if not self.canvas.zoom.show else False
        #self.canvas.zoom.show = True if not self.canvas.zoom.show else False
        self.menu.show = False
    
    def blank(self):
        Input().clear_mouse()
        self.canvas.blank()
        self.menu.show = False
    
    def load(self):
        Input().clear_mouse()
        self.canvas.fm.load()
        self.menu.show = False
    
    def save(self):
        Input().clear_mouse()
        self.canvas.fm.save()
        self.menu.show = False   
        
    def save_as(self):
        Input().clear_mouse()
        self.canvas.fm.save_as()
        self.menu.show = False
    
    # affiche la l'aide
    def launch_about_help(self):
        Input().clear_mouse()
        Input().clear_kb()
        Gui().status = GuiStatus.ABOUT_HELP
        self.about_help.show = True       
        self.menu.show = False