from constantes import GR_AFF, GR_CURS, GR_DRAW, GR_TMP, GR_VARIOUS, GR_ZOOM
from cursors_and_mark import Curs, Mark, Selection
from graphics_shape import Shape
from init import init_gr
from input import Input
from wr_graphics import wr_getpix_p, wr_graphic_to_display, wr_line_p, wr_mouse, wr_triangle_p, wr_txtout_p
from hpprime import strblit2
from files_manager import FilesManager
from math import fabs, sqrt

class Canvas():
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(
                Canvas, cls).__new__(cls)
            # Initialisation des attributs
            cls._instance.initialized = False

        return cls._instance

    def __init__(self):
        if self.initialized:
            return
        self.cur1 = Curs(160, 120)
        self.cur2 = Curs(160, 120)
        self.shp = Shape()
        self.mk = Mark(0, 0, False)
        self.select = Selection(self.mk, self.cur1)
        self.initialized = True
        self.method_mark = ""
        self.zoom = Zoom(GR_ZOOM, 0, 0, 190, 190, 5, GR_CURS)
        self.hud = HUD(GR_VARIOUS, self.shp.edgecolor, self.shp.bgcolor, self.cur1.get_x(), self.cur1.get_y())
        self.fm = FilesManager()

    def draw(self):
        # DRAW->TMP  TMP->CURS CURS->AFF

        # afficher grdraw dans grtmp
        need_redraw = not self._check_mark(self.mk.x, self.mk.y, self.cur1.get_xy(),
                                           self.shp.edgecolor, self.shp.bgcolor,
                                           self.shp.ca_edgecolor, self.shp.ca_bgcolor)
        need_redraw |= Input().kb != 0

        if need_redraw:
            wr_graphic_to_display(
                GR_TMP, 0, 0, GR_DRAW)
            self.shp.tool(
                self.mk, self.cur1)
            

        wr_graphic_to_display(
            GR_VARIOUS, 0, 0, GR_TMP)
        self.hud.set_infos(self.shp.edgecolor, self.shp.bgcolor, self.cur1.get_x(), self.cur1.get_y(), self.mk.x, self.mk.y)
        self.hud.draw()

        wr_graphic_to_display(
            GR_CURS, 0, 0, GR_VARIOUS)
        self.select.display(GR_CURS)
        self.mk.display(GR_CURS)
        self.cur1.display(GR_CURS)

        if self.zoom.show:
            self.zoom.zone(self.cur1)
            wr_graphic_to_display(
                GR_CURS, 0, 0, GR_ZOOM)

        wr_graphic_to_display(
            GR_AFF, 0, 0, GR_CURS)

    def _check_mark(self, *args):
        mark = ""
        for arg in args:
            mark += str(arg)
        if (self.method_mark == mark):
            return True
        self.method_mark = mark
        return False

    def update(self):
        from gui import Gui
        kb = Input().kb
        x1, y1 = self.cur1.get_xy()
        x2, y2 = self.cur2.get_xy()
        if kb == 2**31:
            Gui().select_tool("polyline")
        elif kb == 2**29:
            Gui().load()
        elif kb == 2**23:
            Gui().swap_colors()
        elif kb == 2**21:
            Gui().pick_color()
        elif kb == 2**11:
            self.hud.show = True if not self.hud.show else False
        elif kb == 2**26:
            if self.mk.show:
                self.shp.tool = self.shp.line
        elif kb == 2**16:
            if self.mk.show:
                self.shp.tool = self.shp.circle
        elif kb == 2**15:
            if self.mk.show:
                self.shp.tool = self.shp.rect
        elif kb == 2**20:
            self.shp.tool = self.shp.fill_zone
        elif kb == 2**35:
            Gui().select_input_tool("text_nbg")
        elif kb == (2**35 + 2**41):
            Gui().select_input_tool("text")
        elif kb == 2**30:
            self.shp.validate(GR_DRAW)
            if self.select.show and not self.select.validated:
                self.select.validate()
        elif kb == 2**48:
            Gui().select_tool("point")
        elif kb == 2**40:
            self.mk.x = x1
            self.mk.y = y1
            self.mk.show = True if not self.mk.show else False
            self.select.show = False
            self.shp.tool = self.shp.nop
            self.select.validated = False
        elif kb == 2**17:
            Gui().blank()
            
        elif kb == 2**39 and x1 < 319:
            self.cur1.set_xy(x1+1, y1)

        elif kb == 2**37 and x1 > 0:
            self.cur1.set_xy(x1-1, y1)

        elif kb == 2**43 and y1 < 239:
            self.cur1.set_xy(x1, y1+1)

        elif kb == 2**33 and y1 > 0:
            self.cur1.set_xy(x1, y1-1)

        elif kb == 2**32 and x1 > 0 and y1 > 0:
            self.cur1.set_xy(x1-1, y1-1)

        elif kb == 2**42 and x1 > 0 and y1 < 239:
            self.cur1.set_xy(x1-1, y1+1)

        elif kb == 2**44 and x1 < 319 and y1 < 239:
            self.cur1.set_xy(x1+1, y1+1)

        elif kb == 2**34 and x1 < 319 and y1 > 0:
            self.cur1.set_xy(x1+1, y1-1)
        elif kb == 2**50:
            self.zoom.show = True if not self.zoom.show else False
            
        else:
            f1, f2 = wr_mouse()
            if len(f1) > 0 and len(f2) == 0:
                self.cur1.set_xy(
                    f1[0], f1[1])

            elif len(f1) > 0 and len(f2) > 0:
                self.cur1.set_xy(
                    f1[0], f1[1])
                self.cur1.set_xy(
                    f2[0], f2[1])

    def pick_color(self):
        self.shp.edgecolor = wr_getpix_p(GR_DRAW, self.cur1.x, self.cur1.y)
    
    def blank(self):
        init_gr(GR_DRAW, 320,
                    240, 0xffffff)
        self.fm.reset()

    def selection(self):
        from gui import Gui
        self.select.mark.x = self.mk.x
        self.select.mark.y = self.mk.y
        self.select.mark2.x = self.cur1.x
        self.select.mark2.y = self.cur1.y
        self.select.show = True
        Gui().menu.show = False
    

class Zoom():
    def __init__(self, gr_dest, x, y, width, height, zoom_factor, gr_to_zoom):
        self.gr_dest = gr_dest
        self.x = x
        self.y = y
        self.width = width
        self.height = height
        self.zoom_factor = zoom_factor
        self.gr_to_zoom = gr_to_zoom
        self.show = False

    def zone(self, curs: Curs):
        s_width = self.width // self.zoom_factor
        s_height = self.height // self.zoom_factor

        strblit2(self.gr_dest, self.x, self.y, self.width,
                 self.height, self.gr_to_zoom, curs.x - s_width // 2, curs.y - s_height // 2, s_width, s_height)

    

class HUD():
    def __init__(self, gr, edge_color, bg_color, curs_x, curs_y):
        self.gr = gr
        self.edge_color = edge_color
        self.bg_color = bg_color
        self.curs_x = curs_x
        self.curs_y = curs_y
        self.mark_x = -1
        self.mark_y = -1

        self.show = True
    
    def set_infos(self, edge_color, bg_color, curs_x, curs_y, mark_x = -1, mark_y = -1):
        self.edge_color = edge_color
        self.bg_color = bg_color
        self.curs_x = curs_x
        self.curs_y = curs_y
        self.mark_x = mark_x
        self.mark_y = mark_y

        
    
    def draw_color_indicator(self):
        wr_triangle_p(self.gr, 300, 19, self.bg_color, 319, 19, self.bg_color, 319, 0, self.bg_color, 0) 
        wr_triangle_p(self.gr, 300, 0, self.edge_color, 319, 0, self.edge_color, 300, 19, self.edge_color, 0)
        wr_line_p(self.gr, 300, 19 , 319 ,0, 0x0,0)
        cxcy = "x : "+str(self.curs_x)+" , y : "+str(self.curs_y)
        sep1="dx : "
        dxdy=str(self.curs_x-self.mark_x)+" , dy : "+str(self.curs_y-self.mark_y)
        sep2="dxy : "
        dxy=str(int(sqrt(fabs(self.curs_x-self.mark_x)**2+fabs(self.curs_y-self.mark_y)**2)))
        #wr_txtout_p(cxcy+sep1+dxdy+sep2+dxy,self.gr,0,230,1,0xFF,255,0xFFFFFF)
        wr_txtout_p(cxcy,self.gr,0,230,1,0xFF,255,0xBBBBBB)
        if  Canvas().mk.show : 
            wr_txtout_p(sep1+dxdy,self.gr,0,217,1,0xFF,255,0xFFFF00)
            wr_txtout_p(sep2+dxy,self.gr,0,204,1,0xFF,255,0x80FF00)
        
        
    def draw(self):
        if not self.show:
            return
        self.draw_color_indicator()
        

